@@ -32,7 +32,7 @@
 //!
 //! # Examples
 //!
-//! [`cryo!`], [`Cryo`], and [`LocalLock`] (single-thread lock
+//! [`with_cryo`], [`Cryo`], and [`LocalLock`] (single-thread lock
 //! implementation, used by default):
 //!
 //! ```
@@ -41,10 +41,8 @@
 //!
 //! let cell: usize = 42;
 //!
-//! {
-//!     // `cryo!` uses `LocalLock` by default
-//!     cryo!(let cryo: Cryo<usize> = &cell);
-//!
+//! // `with_cryo` uses `LocalLock` by default
+//! with_cryo(&cell, |cryo: Pin<&Cryo<'_, usize, _>>| {
 //!     // Borrow `cryo` and move it into a `'static` closure.
 //!     let borrow: CryoRef<usize, _> = cryo.borrow();
 //!     let closure: Box<dyn Fn()> =
@@ -58,10 +56,10 @@
 //!     // When `cryo` is dropped, it will block until there are no other
 //!     // references to `cryo`. In this case, the program will leave
 //!     // this block immediately because `CryoRef` has already been dropped.
-//! }
+//! });
 //! ```
 //!
-//! [`cryo!`], [`Cryo`], and [`SyncLock`] (thread-safe lock implementation):
+//! [`with_cryo`], [`Cryo`], and [`SyncLock`] (thread-safe lock implementation):
 //!
 //! ```
 //! # use cryo::*;
@@ -69,10 +67,8 @@
 //!
 //! let cell: usize = 42;
 //!
-//! {
-//!     // This this we are specifying the lock implementation
-//!     cryo!(let cryo: Cryo<usize, SyncLock> = &cell);
-//!
+//! // This time we are specifying the lock implementation
+//! with_cryo((&cell, lock_ty::<SyncLock>()), |cryo| {
 //!     // Borrow `cryo` and move it into a `'static` closure.
 //!     // `CryoRef` can be sent to another thread because
 //!     // `SyncLock` is thread-safe.
@@ -85,18 +81,16 @@
 //!     // When `cryo` is dropped, it will block until there are no other
 //!     // references to `cryo`. In this case, the program will not leave
 //!     // this block until the thread we just spawned completes execution.
-//! }
+//! });
 //! ```
 //!
-//! [`cryo!`], [`CryoMut`], and [`SyncLock`]:
+//! [`with_cryo`], [`CryoMut`], and [`SyncLock`]:
 //!
 //! ```
 //! # use cryo::*;
 //! # use std::{thread::spawn, pin::Pin};
 //! # let mut cell: usize = 0;
-//! {
-//!     cryo!(let cryo_mut: CryoMut<usize, SyncLock> = &mut cell);
-//!
+//! with_cryo((&mut cell, lock_ty::<SyncLock>()), |cryo_mut| {
 //!     // Borrow `cryo_mut` and move it into a `'static` closure.
 //!     let mut borrow: CryoMutWriteGuard<usize, _> = cryo_mut.write();
 //!     spawn(move || { *borrow = 1; });
@@ -104,7 +98,7 @@
 //!     // When `cryo_mut` is dropped, it will block until there are no other
 //!     // references to `cryo_mut`. In this case, the program will not leave
 //!     // this block until the thread we just spawned completes execution
-//! }
+//! });
 //! assert_eq!(cell, 1);
 //! ```
 //!
@@ -117,10 +111,7 @@
 //! // `Cryo` while a `CryoRef` is still referencing it, and `Cryo`'s
 //! // destructor will wait for the `CryoRef` to be dropped first (which
 //! // will never happen)
-//! let borrow = {
-//!     cryo!(let cryo: Cryo<_, SyncLock> = &cell);
-//!     cryo.borrow()
-//! };
+//! let borrow = with_cryo((&cell, lock_ty::<SyncLock>()), |cryo| cryo.borrow());
 //! ```
 //!
 //! ```should_panic
@@ -130,10 +121,7 @@
 //! // `Cryo` while a `CryoRef` is still referencing it, and `Cryo`'s
 //! // destructor will panic, knowing no amount of waiting would cause
 //! // the `CryoRef` to be dropped
-//! let borrow = {
-//!     cryo!(let cryo: Cryo<_> = &cell);
-//!     cryo.borrow()
-//! };
+//! let borrow = with_cryo(&cell, |cryo| cryo.borrow());
 //! ```
 //!
 //! # Caveats
@@ -287,8 +275,43 @@ unsafe impl<T: ?Sized + Send, Lock: crate::Lock> Send for CryoMutWriteGuard<T, L
 
 impl<'a, T: ?Sized + 'a, Lock: crate::Lock> Cryo<'a, T, Lock> {
     /// Construct a new `Cryo`.
+    ///
+    /// # Safety
+    ///
+    /// The created `Cryo` should be dropped before `x` is invalidated. Pinning
+    /// is insufficient as demonstrated below:
+    ///
+    /// ```rust,should_panic
+    /// use cryo::{Cryo, LocalLock};
+    /// use std::{pin::Pin, cell::Cell};
+    ///
+    /// struct OnDrop<F: FnMut()>(u32, F);
+    /// impl<F: FnMut()> Drop for OnDrop<F> {
+    ///     fn drop(&mut self) { (self.1)(); }
+    /// }
+    ///
+    /// let the_box_dropped = Cell::new(false);
+    /// let the_box = Box::new(OnDrop(42, || the_box_dropped.set(true)));
+    ///
+    /// let cryo = Box::pin(unsafe { Cryo::<_, LocalLock>::new(&*the_box) });
+    /// let cryo_ref = Pin::as_ref(&cryo).borrow();
+    ///
+    /// std::mem::forget(cryo);
+    /// drop(the_box);
+    ///
+    /// // `cryo_ref` is still around, but the referenced `*the_box` is gone.
+    /// // Dereferencing `cryo_ref` at this point would cause an undefined
+    /// // behavior.
+    /// // (`*cryo`, referencing the non-existent `*the_box`, is still present
+    /// // in memory as per the pinning guarantee.)
+    /// assert!(!the_box_dropped.get());
+    /// dbg!(cryo_ref.0);
+    /// ```
+    ///
+    /// In version 0.2.2 and earlier, this method was not `unsafe fn` due to an
+    /// oversight.
     #[inline]
-    pub fn new(x: &'a T) -> Self {
+    pub unsafe fn new(x: &'a T) -> Self {
         Self {
             state: UnsafeCell::new(State {
                 data: NonNull::from(x),
@@ -335,8 +358,16 @@ impl<'a, T: ?Sized + 'a, Lock: crate::Lock> Drop for Cryo<'a, T, Lock> {
 
 impl<'a, T: ?Sized + 'a, Lock: crate::Lock> CryoMut<'a, T, Lock> {
     /// Construct a new `CryoMut`.
+    ///
+    /// # Safety
+    ///
+    /// The created `CryoMut` should be dropped before `x` is invalidated.
+    /// Pinning is insufficient as demonstrated in [`Cryo::new`]'s example.
+    ///
+    /// In version 0.2.2 and earlier, this method was not `unsafe fn` due to an
+    /// oversight.
     #[inline]
-    pub fn new(x: &'a mut T) -> Self {
+    pub unsafe fn new(x: &'a mut T) -> Self {
         Self {
             state: UnsafeCell::new(State {
                 data: NonNull::from(x),
@@ -531,15 +562,82 @@ impl<T: ?Sized, Lock: crate::Lock> Drop for CryoMutWriteGuard<T, Lock> {
 
 /// Construct a [`Cryo`] or [`CryoMut`] and bind it to a local variable.
 ///
+/// # Safety
+///
+/// **Don't use. This macro is unsound when used inside an `async fn`.** This
+/// macro doesn't require `unsafe { ... }` merely not to cause breakage.
+///
+/// The unsafety is demonstrated in the following code:
+///
+/// ```should_panic
+/// #![allow(deprecated)]
+/// use cryo::cryo;
+/// use std::{
+///     sync::atomic::{AtomicBool, Ordering},
+///     future::Future,
+///     task::Context,
+/// };
+///
+/// // For demonstration purposes, we want to stop execution when an undefined
+/// // behavior is about occur. To this end, we track the `UserType` object's
+/// // aliveness with this flag.
+/// static IS_USER_TYPE_ALIVE: AtomicBool = AtomicBool::new(true);
+///
+/// struct UserType(u32);
+///
+/// impl Drop for UserType {
+///     fn drop(&mut self) {
+///         IS_USER_TYPE_ALIVE.store(false, Ordering::Relaxed);
+///     }
+/// }
+///
+/// // Let there be a `UserType`.
+/// let user_type = UserType(42);
+///
+/// let mut borrow = None;
+///
+/// // Apply `cryo!` on it inside an `async` block.
+/// let mut fut = Box::pin(async {
+///     cryo!(let cryo: Cryo<_, cryo::SyncLock> = &user_type);
+///
+///     // Leak `borrow` to the outer environment
+///     borrow = Some(cryo.borrow());
+///
+///     // This `Future` will get stuck here. Furthermore, we `forget` this
+///     // `Future`, so `cryo`'s destructor will never run.
+///     std::future::pending::<()>().await
+/// });
+///
+/// // Run the `Future` until it stalls
+/// fut.as_mut().poll(&mut Context::from_waker(&futures::task::noop_waker()));
+///
+/// // Forget the `Future`. The compiler thinks `user_type` is not borrowed,
+/// // but in fact `cryo`, which is borrowing it, is still on memory.
+/// std::mem::forget(fut);
+///
+/// // And `user_type` is gone. Now `cryo` is dangling.
+/// drop(user_type);
+///
+/// // But we can still access the dead `user_type` through `borrow`!
+/// let borrow = borrow.unwrap();
+/// assert!(
+///     IS_USER_TYPE_ALIVE.load(Ordering::Relaxed),
+///     "`cryo!` is supposed to keep us safe, isn't it?"  // well, it betrayed us. (panics)
+/// );
+/// assert_eq!(borrow.0, 42);  // UB
+/// ```
+///
 /// # Examples
 ///
 /// ```
+/// #![allow(deprecated)]
 /// use cryo::cryo;
 /// cryo!(let cryo: Cryo<u8> = &42);
 /// assert_eq!(*cryo.borrow(), 42);
 /// ```
 ///
 /// ```
+/// #![allow(deprecated)]
 /// use cryo::cryo;
 /// let mut var = 42;
 /// {
@@ -553,6 +651,7 @@ impl<T: ?Sized, Lock: crate::Lock> Drop for CryoMutWriteGuard<T, Lock> {
 /// defaults to [`LocalLock`] when unspecified.
 ///
 /// ```
+/// #![allow(deprecated)]
 /// use cryo::cryo;
 /// use std::thread::spawn;
 /// cryo!(let cryo: Cryo<_, cryo::SyncLock> = &42);
@@ -561,6 +660,8 @@ impl<T: ?Sized, Lock: crate::Lock> Drop for CryoMutWriteGuard<T, Lock> {
 ///     assert_eq!(*borrow, 42);
 /// });
 /// ```
+#[deprecated = "`cryo!` is unsound when used inside `async fn` and will be \
+                removed in a future version"]
 #[macro_export]
 macro_rules! cryo {
     // empty (base case for the recursion)
@@ -574,7 +675,7 @@ macro_rules! cryo {
 
     // handle a single declaration
     ($(#[$attr:meta])* let $name:ident: $Cryo:ident< $t:ty $(, $Lock:ty)? > = $init:expr) => (
-        let cryo = $crate::$Cryo::<$t, $crate::__LockOrDefault!($(($Lock))?)>::new($init);
+        let cryo = unsafe { $crate::$Cryo::<$t, $crate::__LockOrDefault!($(($Lock))?)>::new($init) };
         $crate::pin_mut!(cryo);
         let $name = cryo.into_ref();
     );
@@ -592,3 +693,95 @@ macro_rules! __LockOrDefault {
         $crate::LocalLock
     };
 }
+
+/// The trait for types that can be wrapped with [`Cryo`] or [`CryoMut`].
+pub trait WithCryo: private::Sealed + Sized {
+    type Cryo;
+
+    /// Call a given function with a constructed [`Cryo`] or [`CryoMut`].
+    ///
+    /// This method is also exposed as a global function [`with_cryo`].
+    fn with_cryo<R>(self, f: impl FnOnce(Pin<&Self::Cryo>) -> R) -> R;
+}
+
+mod private {
+    pub trait Sealed {}
+    impl<T> Sealed for &T {}
+    impl<T> Sealed for &mut T {}
+    impl<T, Lock> Sealed for (&T, Lock) {}
+    impl<T, Lock> Sealed for (&mut T, Lock) {}
+}
+
+/// Constructs [`Cryo`] with [`LocalLock`] as its [`Lock`] type.
+impl<'a, T> WithCryo for &'a T {
+    type Cryo = Cryo<'a, T, LocalLock>;
+
+    #[inline]
+    fn with_cryo<R>(self, f: impl FnOnce(Pin<&Self::Cryo>) -> R) -> R {
+        let c = unsafe { Self::Cryo::new(self) };
+        pin_mut!(c);
+        f(c.as_ref())
+    }
+}
+
+/// Constructs [`CryoMut`] with [`LocalLock`] as its [`Lock`] type.
+impl<'a, T> WithCryo for &'a mut T {
+    type Cryo = CryoMut<'a, T, LocalLock>;
+
+    #[inline]
+    fn with_cryo<R>(self, f: impl FnOnce(Pin<&Self::Cryo>) -> R) -> R {
+        let c = unsafe { Self::Cryo::new(self) };
+        pin_mut!(c);
+        f(c.as_ref())
+    }
+}
+
+/// Constructs [`Cryo`] with a specified [`Lock`] type.
+impl<'a, T, Lock: crate::Lock> WithCryo for (&'a T, LockTyMarker<Lock>) {
+    type Cryo = Cryo<'a, T, Lock>;
+
+    #[inline]
+    fn with_cryo<R>(self, f: impl FnOnce(Pin<&Self::Cryo>) -> R) -> R {
+        let c = unsafe { Self::Cryo::new(self.0) };
+        pin_mut!(c);
+        f(c.as_ref())
+    }
+}
+
+/// Constructs [`CryoMut`] with a specified [`Lock`] type.
+impl<'a, T, Lock: crate::Lock> WithCryo for (&'a mut T, LockTyMarker<Lock>) {
+    type Cryo = CryoMut<'a, T, Lock>;
+
+    #[inline]
+    fn with_cryo<R>(self, f: impl FnOnce(Pin<&Self::Cryo>) -> R) -> R {
+        let c = unsafe { Self::Cryo::new(self.0) };
+        pin_mut!(c);
+        f(c.as_ref())
+    }
+}
+
+/// Marker type to specify the `Lock` type to use with [`with_cryo`].
+pub struct LockTyMarker<T: ?Sized>(PhantomData<T>);
+
+impl<T: ?Sized> Default for LockTyMarker<T> {
+    #[inline]
+    fn default() -> Self {
+        Self(PhantomData)
+    }
+}
+
+/// Construct a [`LockTyMarker`].
+#[inline]
+pub const fn lock_ty<T>() -> LockTyMarker<T> {
+    LockTyMarker(PhantomData)
+}
+
+/// Call a given function with a constructed [`Cryo`] or [`CryoMut`].
+///
+/// This function is a thin wrapper of [`WithCryo::with_cryo`].
+///
+/// See [the crate documentation](crate) for examples.
+#[inline]
+pub fn with_cryo<T: WithCryo, R>(x: T, f: impl FnOnce(Pin<&T::Cryo>) -> R) -> R {
+    x.with_cryo(f)
+}
